from argparse import ArgumentParser
import sys
from .file_operations import read_config, gen_config
from .misc import greet


def main(argv=sys.argv[1:]):
    print(greet())
    parser = ArgumentParser()
    parser.add_argument(
        "-i",
        "--input",
        help="Input json file for easy installation, currently unsupported",
        nargs="?",
    )
    arg = parser.parse_args()

    if arg.input:
        read_config(arg.i)
    else:
        try:
            res = gen_config()
            if res:
                print()
                print("Your new website was deployed successfully")
                print("Restart nginx to serve it via nginx.")

        except KeyboardInterrupt:
            print("\nBye!")
            exit()


if __name__ == "__main__":
    main()



import re


def is_valid_hostname(hostname):
    if len(hostname) > 255:
        return False
    if hostname[-1] == ".":
        hostname = hostname[:-1]
    allowed = re.compile("(?!-)[A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
    return all(allowed.match(x) for x in hostname.split("."))



# handles all the file operations
import os
from .validation import is_valid_hostname


def read_config(path):
    try:
        with open(path, "r") as f:
            return f.read()
    except OSError:
        print(f"Specified config file {path} could not be read, please make")
        print("sure you have rights to read the file.")
        exit()
    except FileNotFoundError:
        print(f"Specified config file {path} could not be found, please make")
        print("sure you have specified the right path")
        exit()
    except Exception:
        print("There was a problem reading the config file")
        exit()


def gen_config():
    # handling server name
    URL = input(
        "FQDN of your domain, e.g. example.com or subdomain.example.com without http or https:  "
    )
    if URL == "":
        print("URL can not be empty")
        gen_config()
    if not is_valid_hostname(URL):
        print(f"{URL} does not seem to be a valid hostname.")
        gen_config()

    # config path
    PATH = input(
        f"Where should the new config file be saved?, default is /etc/nginx/sites-available/{URL} :  "
    )
    if PATH == "":
        PATH = "/etc/nginx/sites-available/" + URL

    # handling ports
    print(
        "Ports separated by commas(,) in nginx fashion, like 80, [::]:80 etc"
        + os.linesep,
        "For ssl, just add ssl after the port, e.g. 80, 443 ssl, [::]:80, [::]:443 ssl",
        sep="",
    )
    PORT = input("Enter ports here: ")
    if PORT == "":
        PORT = "80"
    PORT = PORT.split(",")

    # root folder
    ROOT = input("Enter the www root path(default /var/www/html):  ")
    if ROOT == "":
        ROOT = "/var/www/html"

    # proxy pass
    PROXY = input("Should this server need to proxy_pass, enter the address here:  ")
    if PROXY == "":
        PROXY = False

    CONFIG = config_lines(url=URL, ports=PORT, root=ROOT, proxy=PROXY)
    return write_config(path=PATH, data=CONFIG)


def config_lines(url, ports, root, proxy=False):
    head = ["server {"]
    head.append(f"  root {root};")
    head.append(f"  server_name {url};")
    for port in ports:
        if not port == "":
            head.append(f"  listen {port};")
    head.append("   location / {")
    head.append("       try_files $uri $uri/;")
    if proxy:
        head.append(f"      proxy_pass {proxy};")
    head.append("   }")
    head.append("}")
    return head


def write_config(path, data: list):
    data = [line + "\n" for line in data]
    try:
        with open(path, "w") as f:
            f.writelines(data)
        return True
    except OSError:
        print(f"Specified config file {path} could not be written, please make")
        print("sure you have rights to read the file.")
        print(*data, sep="\r\n")
        exit()
    except Exception:
        print("There was a problem writing the config file")
        print(*data, sep=os.linesep)
        exit()



import setuptools
import json
import os

with open("README.md", "r") as f:
    long_description = f.read()

with open(
    os.path.join(
        os.path.dirname(__file__),
        "nginx_quickdeploy",
        "version.json"
    ), "r"
) as f:
    version = json.loads(f.read())

setuptools.setup(
    name="nginx_quickdeploy",
    long_description=long_description,
    long_description_content_type="text/markdown",
    version=version["version"],
    packages=setuptools.find_packages(),
    data_files=[("nginx_quickdeploy", ["nginx_quickdeploy/version.json"])],
    include_package_data=True,
    url="https://github.com/regmibijay/nginx-quickdeploy",
    license="MIT",
    author="Bijay Regmi",
    author_email="github@regdelivery.de",
    description="Nginx Quickdeploy",
    install_requires=[],
    entry_points={
        "console_scripts": [
            "quickdeploy=nginx_quickdeploy.main:main",
        ]
    },
    python_requires=">=3.6",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# handling misc func which are not relevant to
# functioning of this program
import json
import os


with open(os.path.join(os.path.dirname(__file__), "version.json"), "r") as f:
    VERSION = json.loads(f.read())


def greet(version=VERSION):
    return f"""
        #####################################################
        ##             Nginx QuickDeploy                   ##
        ##                v{version["version"]}                           ##
        ## https://github.com/regmibijay/nginx-quickdeploy ##
        ##                                                 ##
        #####################################################


        """
